import unittest

import pandas as pd
from pandas.io.parsers import TextFileReader

from pysus.online_data.vaccine import download_covid


class VaccineTestCase(unittest.TestCase):
    def test_Download(self):
        """Careful! this download can take a long time"""
        df = download_covid("BA")
        self.assertIsInstance(df, TextFileReader)


if __name__ == "__main__":
    unittest.main()



"""
Download SINASC data from DATASUS FTP server
Created on 01/11/17
by fccoelho
license: GPL V3 or Later
"""
import os
import warnings
from ftplib import FTP

import pandas as pd

from pysus.online_data import CACHEPATH
from pysus.utilities.readdbc import read_dbc

warnings.filterwarnings("ignore", message=".*initial implementation of Parquet.*")


def download(state, year, cache=True):
    """
    Downloads data directly from Datasus ftp server
    :param state: two-letter state identifier: MG == Minas Gerais
    :param year: 4 digit integer
    :return: pandas dataframe
    """
    assert len(str(year)) == 4
    state = state.upper()
    if year < 1994:
        raise ValueError("SINASC does not contain data before 1994")
    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    if year >= 1996:
        ftp.cwd("/dissemin/publicos/SINASC/NOV/DNRES")
        fname = "DN{}{}.DBC".format(state, year)
    else:
        ftp.cwd("/dissemin/publicos/SINASC/ANT/DNRES")
        fname = "DNR{}{}.DBC".format(state, str(year)[-2:])
    cachefile = os.path.join(CACHEPATH, "SINASC_" + fname.split(".")[0] + "_.parquet")
    if os.path.exists(cachefile):
        df = pd.read_parquet(cachefile)
        return df

    ftp.retrbinary("RETR {}".format(fname), open(fname, "wb").write)
    df = read_dbc(fname, encoding="iso-8859-1")
    if cache:
        df.to_parquet(cachefile)
    os.unlink(fname)
    return df


def get_available_years(state):
    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    ftp.cwd("/dissemin/publicos/SINASC/ANT/DNRES")
    res = ftp.nlst(f"DNR{state}*.*")
    ftp.cwd("/dissemin/publicos/SINASC/NOV/DNRES")
    res += ftp.nlst(f"DN{state}*.*")
    return res



import unittest

import pandas as pd

from pysus.online_data.SINAN import (
    download,
    get_available_years,
    list_diseases,
)


class TestDownload(unittest.TestCase):
    def test_get_sinan_diseases(self):
        dis = list_diseases()
        self.assertIsInstance(dis, list)
        self.assertIn("Tuberculose", dis)

    def test_get_available_years(self):
        res = get_available_years("RJ", "dengue")
        self.assertIsInstance(res, list)
        assert res[0].startswith("DENG")

    def test_download(self):
        df = download("SP", 2018, "Chagas")
        self.assertIsInstance(df, pd.DataFrame)


if __name__ == "__main__":
    unittest.main()



import os
from ftplib import FTP, error_perm
from io import StringIO

# import download as download
import pandas as pd
from dbfread import DBF

from pysus.online_data import CACHEPATH, _fetch_file
from pysus.utilities.readdbc import read_dbc

agravos = {
    "Animais Peçonhentos": "ANIM",
    "Botulismo": "BOTU",
    "Chagas": "CHAG",
    "Chikungunya": "CHIK",
    "Colera": "COLE",
    "Coqueluche": "COQU",
    "Contact Communicable Disease": "ACBI",
    "Acidentes de Trabalho": "ACGR",
    "Dengue": "DENG",
    "Difteria": "DIFT",
    "Esquistossomose": "ESQU",
    "Febre Amarela": "FAMA",
    "Febre Maculosa": "FMAC",
    "Febre Tifoide": "FTIF",
    "Hanseniase": "HANS",
    "Hantavirose": "HANT",
    "Hepatites Virais": "HEPA",
    "Intoxicação Exógena": "IEXO",
    "Leishmaniose Visceral": "LEIV",
    "Leptospirose": "LEPT",
    "Leishmaniose Tegumentar": "LTAN",
    "Malaria": "MALA",
    "Meningite": "MENI",
    "Peste": "PEST",
    "Poliomielite": "PFAN",
    "Raiva Humana": "RAIV",
    "Tétano Acidental": "TETA",
    "Tétano Neonatal": "TETN",
    "Tuberculose": "TUBE",
    "Violência Domestica": "VIOL",
    "Zika": "ZIKA"
}


def list_diseases():
    """List available diseases on SINAN"""
    return list(agravos.keys())


def get_available_years(state, disease):
    """
    Fetch available years for data related to specific disease and state
    :param state: Two letter state symbol, e.g. 'RJ', 'BR' is also possible for national level.
    :param disease: Disease name. See `SINAN.list_diseases` for valid names
    """
    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    ftp.cwd("/dissemin/publicos/SINAN/DADOS/FINAIS")
    # res = StringIO()
    res = ftp.nlst(f"{agravos[disease.title()]}{state}*.dbc")
    return res

def download(state, year, disease, cache=True):
    """
    Downloads SINAN data directly from Datasus ftp server
    :param state: two-letter state identifier: MG == Minas Gerais
    :param year: 4 digit integer
    :disease: Diseases
    :return: pandas dataframe
    """
    try:
        assert disease.title() in agravos
    except AssertionError:
        print(
            f"Disease {disease} is not available in SINAN.\nAvailable diseases: {list_diseases()}"
        )
    year2 = str(year)[-2:].zfill(2)
    state = state.upper()
    if year < 2007:
        raise ValueError("SINAN does not contain data before 2007")
    
    dis_code = agravos[disease.title()]
    fname = f"{dis_code}{state}{year2}.DBC"
    path = "/dissemin/publicos/SINAN/DADOS/FINAIS"
    cachefile = os.path.join(CACHEPATH, "SINAN_" + fname.split(".")[0] + "_.parquet")

    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    ftp.cwd(path)
    
    df = _fetch_file(fname, path, 'DBC')

    if cache:
        df.to_parquet(cachefile)
    if os.path.exists(fname):
        os.unlink(fname)
    return df



u"""
Created on 16/08/16
by fccoelho
license: GPL V3 or Later
"""
import os
import csv
import gzip
from tqdm import tqdm
from io import BytesIO
from tempfile import NamedTemporaryFile

import geopandas as gpd
import pandas as pd
from dbfread import DBF

try:
    from pysus.utilities._readdbc import ffi, lib
except (ImportError, ModuleNotFoundError):
    from _readdbc import ffi, lib


def read_dbc(filename, encoding="utf-8", raw=False):
    """
    Opens a DATASUS .dbc file and return its contents as a pandas
    Dataframe.
    :param filename: .dbc filename
    :param encoding: encoding of the data
    :param raw: Skip type conversion. Set it to True to avoid type conversion errors
    :return: Pandas Dataframe.
    """
    if isinstance(filename, str):
        filename = filename.encode()
    with NamedTemporaryFile(delete=False) as tf:
        dbc2dbf(filename, tf.name.encode())
        try:
            dbf = DBF(tf.name, encoding=encoding, raw=raw)
            df = gpd.GeoDataFrame(list(dbf))
        except ValueError as exc:
            dbf = DBF(tf.name, encoding=encoding, raw=not raw)
            df = gpd.GeoDataFrame(list(dbf))
    os.unlink(tf.name)

    return df


def dbc2dbf(infile, outfile):
    """
    Converts a DATASUS dbc file to a DBF database.
    :param infile: .dbc file name
    :param outfile: name of the .dbf file to be created.
    """
    if isinstance(infile, str):
        infile = infile.encode()
    if isinstance(outfile, str):
        outfile = outfile.encode()
    p = ffi.new("char[]", os.path.abspath(infile))
    q = ffi.new("char[]", os.path.abspath(outfile))

    lib.dbc2dbf([p], [q])

    # print(os.path.exists(outfile))


def read_dbc_geopandas(filename, encoding="utf-8"):
    """
    Opens a DATASUS .dbc file and return its contents as a pandas
    Dataframe, using geopandas
    :param filename: .dbc filename
    :param encoding: encoding of the data
    :return: Pandas Dataframe.
    """
    if isinstance(filename, str):
        filename = filename
    with NamedTemporaryFile(delete=False) as tf:
        out = tf.name + ".dbf"
        dbc2dbf(filename, out)
        dbf = gpd.read_file(out, encoding=encoding).drop("geometry", axis=1)
        df = pd.DataFrame(dbf)
        os.unlink(out)
    os.unlink(tf.name)

    return df

def read_dbc_dbf(filename: str):
    if filename.endswith(('dbc', 'DBC')):
        df = read_dbc(filename, encoding="iso-8859-1")
    elif filename.endswith(("DBF", "dbf")):
            dbf = DBF(filename, encoding="iso-8859-1")
            df = pd.DataFrame(list(dbf))
    return df

def dbf_to_csvgz(filename: str, encoding: str='iso-8859-1'):
    """
    Streams a dbf file to gzipped CSV file. The Gzipped csv will be saved on the same path but with a csv.gz extension.
    :param filename: path to the dbf file
    """
    data = DBF(filename, encoding=encoding, raw=False)
    fn = os.path.splitext(filename)[0] + '.csv.gz'

    with gzip.open(fn, 'wt') as gzf:
        for i, d in tqdm(enumerate(data), desc='Converting',):
            if i == 0:
                csvwriter = csv.DictWriter(gzf, fieldnames=d.keys())
                csvwriter.writeheader()
                csvwriter.writerow(d)
            else:
                csvwriter.writerow(d)





u"""
Created on 21/09/18
by fccoelho
license: GPL V3 or Later
"""
import os
from pathlib import Path
from ftplib import FTP, error_perm
from dbfread import DBF
from pysus.utilities.readdbc import read_dbc
import pandas as pd

CACHEPATH = os.getenv("PYSUS_CACHEPATH", os.path.join(str(Path.home()), "pysus"))

# create pysus cache directory
if not os.path.exists(CACHEPATH):
    os.mkdir(CACHEPATH)


def cache_contents():
    """
    List the files currently cached in ~/pysus
    :return:
    """
    cached_data = os.listdir(CACHEPATH)
    return [os.path.join(CACHEPATH, f) for f in cached_data]


def _fetch_file(fname: str, path: str, ftype: str) -> pd.DataFrame:
    """
    Fetch a single file.
    :return:
    Pandas Dataframe
    """
    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    ftp.cwd(path)
    try:
        ftp.retrbinary("RETR {}".format(fname), open(fname, "wb").write)
    except:
        raise Exception("File {} not available".format(fname))
    if ftype == "DBC":
        df = read_dbc(fname, encoding="iso-8859-1")
    elif ftype == "DBF":
        dbf = DBF(fname, encoding="iso-8859-1")
        df = pd.DataFrame(list(dbf))

    if os.path.exists(fname):
        os.unlink(fname)
    return df


def get_CID10_table(cache=True):
    """
    Fetch the CID10 table
    :param cache:
    :return:
    """
    fname = "CID10.DBF"
    cachefile = os.path.join(CACHEPATH, "SIM_" + fname.split(".")[0] + "_.parquet")
    if os.path.exists(cachefile):
        df = pd.read_parquet(cachefile)
        return df
    df = _fetch_file(fname, "/dissemin/publicos/SIM/CID10/TABELAS", "DBF")
    if cache:
        df.to_parquet(cachefile)
    return df



u"""
Created on 17/08/16
by fccoelho
license: GPL V3 or Later
"""

import unittest

import pandas as pd

from pysus.utilities.readdbc import read_dbc


class TestReadDbc(unittest.TestCase):
    def test_read_dbc(self):
        df = read_dbc(b"test_data/sids.dbc")
        self.assertIsInstance(df, pd.DataFrame)
        self.assertGreater(df.size, 0)



"""
Fetches the geographical base for demographical analyses.
All data comes from IBGE
"""

import lzma
import os
import tarfile
import tempfile
from multiprocessing import Pool

import colorcet
import dask
import datashader as ds
import geobr
import geopandas as gpd
import georasters as gr
import numpy as np
import wget
from shapely import geometry

LEVELS = {
    "Country": geobr.read_country,
    "Region": geobr.read_region,
    "State": geobr.read_state,
    "Meso region": geobr.read_meso_region,
    "Micro region": geobr.read_micro_region,
    "Immediate region": geobr.read_immediate_region,
    "Census weighting area": geobr.read_weighting_area,
    "Census tract": geobr.read_census_tract,
    "Municipality": geobr.read_municipality,
    "Municipality seats": geobr.read_municipal_seat,
    "Metropolitan areas": geobr.read_metro_area,
    "Urban footprints": geobr.read_urban_area,
    "Brazil's Legal Amazon": geobr.read_amazon,
    "Biomes": geobr.read_biomes,
    "Environmental Conservation Units": geobr.read_conservation_units,
    "Disaster risk areas": geobr.read_disaster_risk_area,
    "Indigenous lands": geobr.read_indigenous_land,
    "Semi Arid region": geobr.read_semiarid,
    "Health facilities": geobr.read_health_facilities,
    "Health regions": geobr.read_health_region,
    "Neighborhood limits": geobr.read_neighborhood,
}


class GeoBase:
    """
    Parameterized geographical base
    """

    def __init__(self, level, reset=False):
        """
        Initialize geographical base at the specified level
        :param level: One of the valid levels specified in `pysus.demography.geobase.LEVELS`
        :param reset: reset the local directory cache. Set to True if you you are changing the parameters of the map.
        default: False
        """
        if reset:
            if os.path.exists(f"{level}_map.parquet"):
                os.remove(f"{level}_map.parquet")
            if os.path.exists(f"{level}_pop.parquet"):
                os.remove(f"{level}_pop.parquet")
            if os.path.exists(f"{level}_raster.parquet"):
                os.remove(f"{level}_raster.parquet")
        else:
            if os.path.exists(f"{level}_map.parquet"):
                print(
                    "You have cached data for this level. Please set `reset=True` if you want to download fresh data"
                )
        try:
            assert level in LEVELS
        except AssertionError:
            print(f"Please select one of these levels: {', '.join(LEVELS.keys())}")
        self.level = level
        self.mapdf = None
        self.pop = None
        self.pop_raster = None

    def __str__(self):
        return f"{self.level} level Geobase"

    def _persist(self, what):
        if what == "map":
            self.mapdf.to_parquet(f"{self.level}_map.parquet")
        elif what == "pop":
            self.pop.to_parquet(f"{self.level}_pop.parquet")
        elif what == "raster":
            self.raster.to_parquet(f"{self.level}_raster.parquet")

    def help_fetch_map(self):
        return help(LEVELS[self.level])

    def map(self, *args, **kwargs):
        """
        Fetches map of `self.level` given parameters
        :param args: positional parameters for geobr map reading function
        :param kwargs: keyword parameters for geobr map reading function
        :return: GeoDataFrame
        """
        if os.path.exists(f"{self.level}_map.parquet"):
            self.mapdf = gpd.read_parquet(f"{self.level}_map.parquet")
            return self.mapdf
        print("Dowloading the Map...")
        if self.mapdf is None:
            self.mapdf = LEVELS[self.level](*args, **kwargs)
            self._persist("map")
        return self.mapdf

    def plot_pop(self, **kwargs):
        """
        Plots a chropletic representation of the population
        :param kwargs: Additional parameters passed to geopandas plot command
        """
        self.mapdf.plot(column="population", **kwargs)

    def demographics(self):
        """
        Adds population data to geoographical base
        :return:
        """
        if "population" in self.mapdf.columns:
            return
        print("Fetching population data...")
        bbox = self.mapdf.to_crs("EPSG:4326").total_bounds
        # raster = fetch_gpw4_raster(bbox)
        self.pop_raster = raster = get_full_pop_raster()
        self.mapdf["population"] = [
            raster.clip(geom)[0].sum() for geom in self.mapdf.geometry
        ]
        self._persist("map")

    def generate_populations(self, scale=0.05):
        """
        Generate a synthetic population of size scale*population size for each polygon in self.mapdf
        :param scale:
        """
        if os.path.exists(f"{self.level}_pop.parquet"):
            self.pop = gpd.read_parquet(f"{self.level}_pop.parquet")
            return
        if "population" not in self.mapdf.columns:
            self.demographics()
        for row in self.mapdf.itertuples():
            people = sample_random_people(int(row.population * scale), row.geometry)
            sex = np.random.randint(0, 2, size=len(people))
            age = np.random.randint(0, 100, size=len(people))
        print(len(people), people[0])
        self.pop = gpd.GeoDataFrame({"sex": sex, "age": age, "geometry": people})
        self.pop["longitude"] = [pt.x for pt in self.pop.geometry]
        self.pop["latitude"] = [pt.y for pt in self.pop.geometry]
        self._persist("pop")

    def plot_synthetic_pop(self):
        canvas = ds.Canvas(plot_width=800, plot_height=600)
        agg = canvas.points(self.pop, x="longitude", y="latitude")
        return ds.tf.shade(agg, cmap=colorcet.fire, how="log")


def contains(args):
    polygon, point = args
    pt = geometry.Point(point)
    return pt, polygon.contains(pt)


def sample_random_people(n, polygon, overestimate=1.5):
    print(f"Synthesizing {n} individuals")
    min_x, min_y, max_x, max_y = polygon.bounds
    ratio = polygon.area / polygon.envelope.area
    samples = np.random.uniform(
        (min_x, min_y), (max_x, max_y), (int((n / ratio) * overestimate), 2)
    )[:n, :]
    # multipoint = geometry.MultiPoint(samples)
    # multipoint = multipoint.intersection(polygon)
    po = Pool()
    res = po.map(contains, ((polygon, p) for p in samples))
    pts = [p for p, c in res if c]  # List of inscribed points
    po.terminate()
    po.join()

    return pts


def get_population(geometry, raster):
    return raster.clip(geometry)[0].sum()


def get_full_pop_raster(path="."):
    url = "https://www.dropbox.com/s/l9iphmawfjzt4lf/brazil_pop.tif.tar.xz?dl=1"
    fn = os.path.join(path, "brazil_pop.tif.tar.xz")
    wget.download(url=url, out=path)
    fn = os.path.join(path, "brazil_pop.tif.tar.xz")
    with lzma.open("brazil_pop.tif.tar.xz") as f:
        with tarfile.open(fileobj=f) as tar:
            tar.extractall()
            # with open('brazil_pop.tif', 'wb') as brr:
            #     brr.write(tar.extractall(path=path))
    os.unlink("brazil_pop.tif.tar.xz")
    raster = gr.from_file("brazil_pop.tif.tif")
    os.unlink("brazil_pop.tif.tif")

    return raster



import datetime
import unittest

import numpy as np
import pandas as pd

from pysus.online_data.SINAN import download, list_diseases
from pysus.preprocessing.sinan import geocode, read_sinan_dbf


class TestSINANDownload(unittest.TestCase):
    def test_download(self):
        df = download("BR", 2007, "Animais Peçonhentos")
        self.assertIsInstance(df, pd.DataFrame)

    def test_fetch_viol_dom(self):
        df = download("BR", 2011, "Violência Domestica")
        self.assertIsInstance(df, pd.DataFrame)

    def test_lista_agravos(self):
        lista = list_diseases()
        self.assertIsInstance(lista, list)
        self.assertGreater(len(lista), 0)


class TestSinanDF(unittest.TestCase):
    def test_read_dbf(self):
        df = read_sinan_dbf("test_data/EPR-2016-06-01-2016.dbf", encoding="latin-1")
        self.assertIsInstance(df, pd.DataFrame)
        for cname in df.columns:
            if cname.startswith("DT_"):
                self.assertIsInstance(df[cname][0], datetime.date)
            elif cname.startswith("SEM"):
                self.assertLessEqual(df[cname][0], 52)
                self.assertIsInstance(df[cname][0], (int, np.int64))
            elif cname.startswith(("NU", "ID")):
                if cname == "ID_AGRAVO":
                    continue
                self.assertIsInstance(
                    df[cname][0],
                    (int, float, np.int64),
                    msg="Failed on column {}, type:{}".format(
                        cname, type(df[cname][0])
                    ),
                )

    def test_type_convertion(self):
        df = read_sinan_dbf("test_data/EPR-2016-06-01-2016.dbf", encoding="latin-1")
        assert not all(df.dtypes == "object")

    def test_geocode(self):
        df = pd.read_pickle("test_data/chik.pickle")

    #  geocode(sinan_df=df, outfile='chik_2016.csv', default_city='Rio de Janeiro')


if __name__ == "__main__":
    unittest.main()



u"""
Created on 16/08/16
by fccoelho
license: GPL V3 or Later
"""
import os

from cffi import FFI

ffibuilder = FFI()

PATH = os.path.dirname(__file__)

with open(os.path.join(PATH, "c-src/dbc2dbf.c"), "r") as f:
    ffibuilder.set_source(
        "_readdbc",
        f.read(),
        libraries=["c"],
        sources=[os.path.join(PATH, "c-src/blast.c")],
        include_dirs=[os.path.join(PATH, "c-src/")],
    )
ffibuilder.cdef(
    """
    static unsigned inf(void *how, unsigned char **buf);
    static int outf(void *how, unsigned char *buf, unsigned len);
    void dbc2dbf(char** input_file, char** output_file);
    """
)

with open(os.path.join(PATH, "c-src/blast.h")) as f:
    ffibuilder.cdef(f.read(), override=True)

if __name__ == "__main__":
    ffibuilder.compile(
        tmpdir=".", target="pysus/utilities/_readdbc.so", verbose=False, debug=None
    )



import os
import time
from datetime import date

import dask.dataframe as dd
import elasticsearch.helpers
import pandas as pd
from elasticsearch import Elasticsearch

from pysus.online_data import CACHEPATH


def download(uf, cache=True, checkmemory=True):
    """
    Download ESUS data by UF
    :param uf: rj, mg, etc
    :param cache: if results should be cached on disk
    :return: DataFrame if data fits in memory, other an iterator of chunks of size 1000.
    """
    uf = uf.lower()
    user = "user-public-notificacoes"
    pwd = "Za4qNXdyQNSa9YaA"
    today = date.today()
    dt = today.strftime("_%d_%m_%Y")
    base = f"desc-notificacoes-esusve-{uf}"  # desc-notificacoes-esusve-
    url = f"https://{user}:{pwd}@elasticsearch-saps.saude.gov.br"
    out = f"ESUS_{uf}_{dt}.parquet"

    cachefile = os.path.join(CACHEPATH, out)
    tempfile = os.path.join(CACHEPATH, f"ESUS_temp_{uf.upper()}.csv.gz")
    if os.path.exists(cachefile):
        df = pd.read_parquet(cachefile)
    elif os.path.exists(tempfile):
        df = pd.read_csv(tempfile, chunksize=1000)
    else:
        fname = fetch(base, uf, url)
        size = os.stat(fname).st_size
        if size > 50e6 and checkmemory:
            print(f"Downloaded data is to large:{size / 1e6} MB compressed.")
            print(
                "Only loading the first 1000 rows. If your computer has enough memory, set 'checkmemory' to False"
            )
            print(f"The full data is in {fname}")
            df = pd.read_csv(fname, chunksize=1000)
        else:
            df = pd.read_csv(fname, low_memory=False)
            print(f"{df.shape[0]} records downloaded.")
            os.unlink(fname)
            if cache:
                df.to_parquet(cachefile)

    return df


def fetch(base, uf, url):
    UF = uf.upper()
    print(f"Reading ESUS data for {UF}")
    es = Elasticsearch([url], send_get_body_as="POST")
    body = {"query": {"match_all": {}}}
    results = elasticsearch.helpers.scan(es, query=body, index=base)
    # df = pd.DataFrame.from_dict([document['_source'] for document in results])

    chunker = chunky_fetch(results, 3000)
    h = 1
    tempfile = os.path.join(CACHEPATH, f"ESUS_temp_{UF}.csv.gz")
    for ch in chunker:
        df = pd.DataFrame.from_dict(ch)
        df.sintomas = df["sintomas"].str.replace(
            ";",
            "",
        )  ## remove os  ;
        if h:
            df.to_csv(tempfile)
            h = 0
        else:
            df.to_csv(tempfile, mode="a", header=False)
    # df = pd.read_csv('temp.csv.gz')

    return tempfile


def chunky_fetch(results, chunk_size=3000):
    "Fetches data in chunks to preserve memory"
    data = []
    i = 0
    for d in results:
        data.append(d["_source"])
        i += 1
        if i == chunk_size:
            yield data
            data = []
            i = 0
    else:
        return data



from setuptools import find_packages, setup

ld = """
PySUS
=====

This package collects a set of utilities for handling with public databases
published by Brazil's DATASUS
The documentation of how to use PySUS can be found
here: http://pysus.readthedocs.io/en/latest/

Features
--------

- Decode encoded patient age to any time unit (years, months, etc)
- Convert `.dbc` files to DBF databases or read them into pandas dataframes.
  DBC files are basically DBFs compressed by a proprietary algorithm.
- Loads SINAN DBC files into Pandas Dataframes
- Downloads SIA, SIH, SIM, SINASC, and ESUS (covid data)
- Geocodes SINAN notified cases in batch. You can use your Google API KEY
  to avoid Google's free limits.

Installation
------------
Make sure your system has libffi-dev package installed::

$ sudo pip install PySUS

"""

with open("requirements.txt") as f:
    requirements = f.readlines()

test_requirements = ["pytest", "flake8"]
dev_requirements = []
dev_requirements += requirements

setup(
    name="PySUS",
    version="0.5.13",
    packages=find_packages(),
    package_data={"": ["*.c", "*.h", "*.o", "*.so", "*.md", "*.txt"]},
    include_package_data=True,
    zip_safe=False,
    url="https://github.com/fccoelho/PySUS",
    license="gpl-v3",
    author="Flavio Codeco Coelho",
    author_email="fccoelho@gmail.com",
    description="Tools for dealing with Brazil's Public health data",
    long_description=ld,
    setup_requires=["cffi>=1.0.0", "setuptools>26.0.0"],
    cffi_modules=["pysus/utilities/_build_readdbc.py:ffibuilder"],
    install_requires=requirements,
    # cmdclass={'install': PostInstall},
    extras_require={"dev": dev_requirements},
    test_suite="tests",
    tests_require=test_requirements,
)



"""
Downloads SIH data from Datasus FTP server
Created on 21/09/18
by fccoelho
license: GPL V3 or Later
"""

import os
from ftplib import FTP

import pandas as pd
from dbfread import DBF

from pysus.online_data import CACHEPATH
from pysus.utilities.readdbc import read_dbc


def download(state: str, year: int, month: int, cache: bool = True) -> object:
    """
    Download SIH records for state year and month and returns dataframe
    :param month: 1 to 12
    :param state: 2 letter state code
    :param year: 4 digit integer
    :param cache: Whether to cache or not. defaults to True.
    :return:
    """
    state = state.upper()
    year2 = int(str(year)[-2:])
    year2 = str(year2).zfill(2)
    month = str(month).zfill(2)
    if year < 1992:
        raise ValueError("SIH does not contain data before 1994")
    if year < 2008:
        ftype = "DBC"
        path = "/dissemin/publicos/SIHSUS/199201_200712/Dados"
        fname = f"RD{state}{year2}{month}.dbc"
    if year >= 2008:
        ftype = "DBC"
        path = f"/dissemin/publicos/SIHSUS/200801_/Dados"
        fname = f"RD{state}{year2}{month}.dbc"
    cachefile = os.path.join(CACHEPATH, "SIH_" + fname.split(".")[0] + "_.parquet")
    if os.path.exists(cachefile):
        df = pd.read_parquet(cachefile)
        return df

    df = _fetch_file(fname, path, ftype)
    if cache:
        df.to_parquet(cachefile)
    return df


def _fetch_file(fname, path, ftype):
    ftp = FTP("ftp.datasus.gov.br")
    ftp.login()
    ftp.cwd(path)
    try:
        ftp.retrbinary("RETR {}".format(fname), open(fname, "wb").write)
    except:
        raise Exception("File {} not available".format(fname))
    if ftype == "DBC":
        df = read_dbc(fname, encoding="iso-8859-1")
    elif ftype == "DBF":
        dbf = DBF(fname, encoding="iso-8859-1")
        df = pd.DataFrame(list(dbf))
    os.unlink(fname)
    return df



import os
import unittest
from time import time

import geopandas as gpd

from pysus.demography import geobase


class Geobase(unittest.TestCase):
    def tearDown(self) -> None:
        if os.path.exists("Municipality_map.parquet"):
            os.unlink("Municipality_map.parquet")
        if os.path.exists("State_map.parquet"):
            os.unlink("State_map.parquet")

    def test_levels(self):
        self.assertIn("Municipality", geobase.LEVELS)

    def test_get_map(self):
        GB = geobase.GeoBase(level="Municipality")
        gdf = GB.map(4108304)
        self.assertIsInstance(gdf, gpd.GeoDataFrame)
        t0 = time()
        gdf = GB.map()  # from second time it should skip download
        tf = time()
        self.assertLess(tf - t0, 1)
        self.assertEqual(int(gdf["code_muni"].values[0]), 4108304)
        self.assertEqual(len(gdf), 1)

    def test_get_map_2019(self):
        GB = geobase.GeoBase(level="Municipality", reset=True)
        gdf = GB.map(4220000, year=2019)
        self.assertIsInstance(gdf, gpd.GeoDataFrame)
        t0 = time()
        gdf = GB.map()  # from second time it should skip download
        tf = time()
        self.assertLess(tf - t0, 1)
        self.assertEqual(int(gdf["code_muni"].values[0]), 4220000)
        self.assertEqual(len(gdf), 1)

    def test_demographics(self):
        GB = geobase.GeoBase(level="State")
        gdf = GB.map("all")
        self.assertEqual(len(gdf), 27)
        GB.demographics()
        self.assertIn("population", GB.mapdf)


if __name__ == "__main__":
    unittest.main()
